@@ -330,7 +330,7 @@ static const int dnsnameraw_en_main = 1;
         const char* eof = pe;
         int cs;
         char val = 0;
-        char labellen=0;
+        unsigned char labellen=0;
         unsigned int lenpos=0;
         
 #line 337 "dnslabeltext.cc"
@@ -414,7 +414,7 @@ _match:
 	case 0:
 #line 114 "dnslabeltext.rl"
 	{ 
-                        if (labellen < 0 || labellen > 63) {
+                        if (labellen > 63) {
                           throw runtime_error("Unable to parse DNS name '"+string(realinput)+"': invalid label length "+std::to_string(labellen));
                         }
                         ret[lenpos]=labellen;
@@ -479,7 +479,7 @@ _again:
 	case 0:
 #line 114 "dnslabeltext.rl"
 	{ 
-                        if (labellen < 0 || labellen > 63) {
+                        if (labellen > 63) {
                           throw runtime_error("Unable to parse DNS name '"+string(realinput)+"': invalid label length "+std::to_string(labellen));
                         }
                         ret[lenpos]=labellen;
