@@ -62,7 +62,9 @@ void Logger::log(const string& msg, Urgency u) noexcept
       time_t t;
       time(&t);
       localtime_r(&t, &tm);
-      strftime(buffer, sizeof(buffer), "%b %d %H:%M:%S ", &tm);
+      if (strftime(buffer, sizeof(buffer), "%b %d %H:%M:%S ", &tm) == 0) {
+        buffer[0] = '\0';
+      }
     }
 
     string prefix;
@@ -200,3 +202,15 @@ Logger& Logger::operator<<(const ComboAddress& ca)
   *this << ca.toLogString();
   return *this;
 }
+
+void addTraceTS(const timeval& start, ostringstream& str)
+{
+  const auto& content = str.str();
+  if (content.empty() || content.back() == '\n') {
+    timeval time{};
+    gettimeofday(&time, nullptr);
+    auto elapsed = time - start;
+    auto diff = elapsed.tv_sec * 1000000 + static_cast<time_t>(elapsed.tv_usec);
+    str << diff << ' ';
+  }
+}
