@@ -52,22 +52,24 @@ using fcontext_t = boost::context::fcontext_t*;
 
 /* Emulate the >= 1.56 API for Boost 1.52 through 1.55 */
 static inline intptr_t
-jump_fcontext (fcontext_t* const ofc, fcontext_t const nfc, 
-               intptr_t const arg) {
-    /* If the fcontext_t is preallocated then use it, otherwise allocate one
-     * on the stack ('self') and stash a pointer away in *ofc so the returning
-     * MThread can access it. This is safe because we're suspended, so the
-     * context object always outlives the jump.
-     */
-    if (*ofc) {
-        return boost::context::jump_fcontext (*ofc, nfc, arg);
-    } else {
-        boost::context::fcontext_t self;
-        *ofc = &self;
-        auto ret = boost::context::jump_fcontext (*ofc, nfc, arg);
-        *ofc = nullptr;
-        return ret;
-    }
+jump_fcontext(fcontext_t* const ofc, fcontext_t const nfc,
+              intptr_t const arg)
+{
+  /* If the fcontext_t is preallocated then use it, otherwise allocate one
+   * on the stack ('self') and stash a pointer away in *ofc so the returning
+   * MThread can access it. This is safe because we're suspended, so the
+   * context object always outlives the jump.
+   */
+  if (*ofc) {
+    return boost::context::jump_fcontext(*ofc, nfc, arg);
+  }
+  else {
+    boost::context::fcontext_t self;
+    *ofc = &self;
+    auto ret = boost::context::jump_fcontext(*ofc, nfc, arg);
+    *ofc = nullptr;
+    return ret;
+  }
 }
 #else
 
@@ -80,28 +82,31 @@ using boost::context::detail::jump_fcontext;
 using boost::context::detail::transfer_t;
 #endif /* BOOST_VERSION < 106100 */
 
-static_assert (std::is_pointer<fcontext_t>::value,
-               "Boost Context has changed the fcontext_t type again :-(");
+static_assert(std::is_pointer<fcontext_t>::value,
+              "Boost Context has changed the fcontext_t type again :-(");
 #endif
 
 /* Boost context only provides a means of passing a single argument across a
  * jump. args_t simply provides a way to pass more by reference.
  */
-struct args_t {
+struct args_t
+{
 #if BOOST_VERSION < 106100
-    fcontext_t prev_ctx = nullptr;
+  fcontext_t prev_ctx = nullptr;
 #endif
-    pdns_ucontext_t* self = nullptr;
-    std::function<void(void)>* work = nullptr;
+  pdns_ucontext_t* self = nullptr;
+  std::function<void(void)>* work = nullptr;
 };
 
-extern "C" {
-static
-void
+extern "C"
+{
+  static void
 #if BOOST_VERSION < 106100
-threadWrapper (intptr_t const xargs) {
+  threadWrapper(intptr_t const xargs)
+  {
 #else
-threadWrapper (transfer_t const t) {
+  threadWrapper(transfer_t const t)
+  {
 #endif
     /* Access the args passed from pdns_makecontext, and copy them directly from
      * the calling stack on to ours (we're now using the MThreads stack).
@@ -123,10 +128,10 @@ threadWrapper (transfer_t const t) {
     /* we switch back to pdns_makecontext() */
     notifyStackSwitchToKernel();
 #if BOOST_VERSION < 106100
-    jump_fcontext (reinterpret_cast<fcontext_t*>(&ctx->uc_mcontext),
-                   static_cast<fcontext_t>(args->prev_ctx), 0);
+    jump_fcontext(reinterpret_cast<fcontext_t*>(&ctx->uc_mcontext),
+                  static_cast<fcontext_t>(args->prev_ctx), 0);
 #else
-    transfer_t res = jump_fcontext (t.fctx, 0);
+    transfer_t res = jump_fcontext(t.fctx, 0);
     /* we got switched back from pdns_swapcontext() */
     if (res.data) {
       /* if res.data is not a nullptr, it holds a pointer to the context
@@ -140,41 +145,43 @@ threadWrapper (transfer_t const t) {
     args = nullptr;
 
     try {
-        auto start = std::move (*work);
-        start();
-    } catch (...) {
-        ctx->exception = std::current_exception();
+      auto start = std::move(*work);
+      start();
+    }
+    catch (...) {
+      ctx->exception = std::current_exception();
     }
 
     notifyStackSwitchToKernel();
     /* Emulate the System V uc_link feature. */
     auto const next_ctx = ctx->uc_link->uc_mcontext;
 #if BOOST_VERSION < 106100
-    jump_fcontext (reinterpret_cast<fcontext_t*>(&ctx->uc_mcontext),
-                   static_cast<fcontext_t>(next_ctx),
-                   reinterpret_cast<intptr_t>(ctx));
+    jump_fcontext(reinterpret_cast<fcontext_t*>(&ctx->uc_mcontext),
+                  static_cast<fcontext_t>(next_ctx),
+                  reinterpret_cast<intptr_t>(ctx));
 #else
-    jump_fcontext (static_cast<fcontext_t>(next_ctx), 0);
+    jump_fcontext(static_cast<fcontext_t>(next_ctx), 0);
 #endif
 
 #ifdef NDEBUG
     __builtin_unreachable();
 #endif
-}
+  }
 }
 
-pdns_ucontext_t::pdns_ucontext_t
-(): uc_mcontext(nullptr), uc_link(nullptr) {
+pdns_ucontext_t::pdns_ucontext_t() :
+  uc_mcontext(nullptr), uc_link(nullptr)
+{
 #ifdef PDNS_USE_VALGRIND
   valgrind_id = 0;
 #endif /* PDNS_USE_VALGRIND */
 }
 
-pdns_ucontext_t::~pdns_ucontext_t
-() {
-    /* There's nothing to delete here since fcontext doesn't require anything
-     * to be heap allocated.
-     */
+pdns_ucontext_t::~pdns_ucontext_t()
+{
+  /* There's nothing to delete here since fcontext doesn't require anything
+   * to be heap allocated.
+   */
 #ifdef PDNS_USE_VALGRIND
   if (valgrind_id != 0) {
     VALGRIND_STACK_DEREGISTER(valgrind_id);
@@ -182,21 +189,20 @@ pdns_ucontext_t::~pdns_ucontext_t
 #endif /* PDNS_USE_VALGRIND */
 }
 
-void
-pdns_swapcontext
-(pdns_ucontext_t& __restrict octx, pdns_ucontext_t const& __restrict ctx) {
+void pdns_swapcontext(pdns_ucontext_t& __restrict octx, pdns_ucontext_t const& __restrict ctx)
+{
   /* we either switch back to threadwrapper() if it's the first time,
      or we switch back to pdns_swapcontext(),
      in both case we will be returning from a call to jump_fcontext(). */
 #if BOOST_VERSION < 106100
-    intptr_t ptr = jump_fcontext(reinterpret_cast<fcontext_t*>(&octx.uc_mcontext),
-                                 static_cast<fcontext_t>(ctx.uc_mcontext), 0);
+  intptr_t ptr = jump_fcontext(reinterpret_cast<fcontext_t*>(&octx.uc_mcontext),
+                               static_cast<fcontext_t>(ctx.uc_mcontext), 0);
 
-    auto origctx = reinterpret_cast<pdns_ucontext_t*>(ptr);
-    if(origctx && origctx->exception)
-        std::rethrow_exception (origctx->exception);
+  auto origctx = reinterpret_cast<pdns_ucontext_t*>(ptr);
+  if (origctx && origctx->exception)
+    std::rethrow_exception(origctx->exception);
 #else
-  transfer_t res = jump_fcontext (static_cast<fcontext_t>(ctx.uc_mcontext), &octx.uc_mcontext);
+  transfer_t res = jump_fcontext(static_cast<fcontext_t>(ctx.uc_mcontext), &octx.uc_mcontext);
   if (res.data) {
     /* if res.data is not a nullptr, it holds a pointer to the context
        we just switched from, and we need to fill it to be able to
@@ -205,34 +211,32 @@ pdns_swapcontext
     *ptr = res.fctx;
   }
   if (ctx.exception) {
-    std::rethrow_exception (ctx.exception);
+    std::rethrow_exception(ctx.exception);
   }
 #endif
 }
 
-void
-pdns_makecontext
-(pdns_ucontext_t& ctx, std::function<void(void)>& start) {
-    assert (ctx.uc_link);
-    assert (ctx.uc_stack.size() >= 8192);
-    assert (!ctx.uc_mcontext);
-    ctx.uc_mcontext = make_fcontext (&ctx.uc_stack[ctx.uc_stack.size()-1],
-                                     ctx.uc_stack.size()-1, &threadWrapper);
-    args_t args;
-    args.self = &ctx;
-    args.work = &start;
-    /* jumping to threadwrapper */
-    notifyStackSwitch(&ctx.uc_stack[ctx.uc_stack.size()-1], ctx.uc_stack.size()-1);
+void pdns_makecontext(pdns_ucontext_t& ctx, std::function<void(void)>& start)
+{
+  assert(ctx.uc_link);
+  assert(ctx.uc_stack.size() >= 8192);
+  assert(!ctx.uc_mcontext);
+  ctx.uc_mcontext = make_fcontext(&ctx.uc_stack[ctx.uc_stack.size() - 1],
+                                  ctx.uc_stack.size() - 1, &threadWrapper);
+  args_t args;
+  args.self = &ctx;
+  args.work = &start;
+  /* jumping to threadwrapper */
+  notifyStackSwitch(&ctx.uc_stack[ctx.uc_stack.size() - 1], ctx.uc_stack.size() - 1);
 #if BOOST_VERSION < 106100
-    jump_fcontext (reinterpret_cast<fcontext_t*>(&args.prev_ctx),
-                   static_cast<fcontext_t>(ctx.uc_mcontext),
-                   reinterpret_cast<intptr_t>(&args));
+  jump_fcontext(reinterpret_cast<fcontext_t*>(&args.prev_ctx),
+                static_cast<fcontext_t>(ctx.uc_mcontext),
+                reinterpret_cast<intptr_t>(&args));
 #else
-    transfer_t res = jump_fcontext (static_cast<fcontext_t>(ctx.uc_mcontext),
-                                    &args);
-    /* back from threadwrapper, updating the context */
-    ctx.uc_mcontext = res.fctx;
+  transfer_t res = jump_fcontext(static_cast<fcontext_t>(ctx.uc_mcontext),
+                                 &args);
+  /* back from threadwrapper, updating the context */
+  ctx.uc_mcontext = res.fctx;
 #endif
-    notifyStackSwitchDone();
-
+  notifyStackSwitchDone();
 }
